@@ -2,7 +2,7 @@
 #   pytest -v test_hapitime2datetime.py
 
 # To test a single function on the command line, use, e.g.,
-#   pytest -v  est_hapitime2datetime.py -k test_parse_string_input
+#   pytest -v test_hapitime2datetime.py -k test_parse_string_input
 
 # To use in program, use, e.g.,
 #   from hapiclient.test.test_hapitime2datetime import test_parse_string_input
@@ -15,7 +15,7 @@ from hapiclient import hapitime2datetime
 from hapiclient.util import log
 
 # Create empty file
-with open("test_hapitime2datetime.log", "w") as f: pass
+with open(os.path.realpath(__file__)[0:-2] + "log", "w") as f: pass
 logging = open(os.path.realpath(__file__)[0:-2] + "log", "a")
 
 expected = '1970-01-01T00:00:00.000000Z'
